% InSegtFibre_2D
% Written by Monica Jane Emerson, April 2018.
% MIT license

%% Step 0: Add paths of GUI and other functions and scripts used in Insegt Fibre
close all,
clear all,
addpath(genpath('./texture_gui'))
addpath('./scripts')
addpath('./functions')

disp('Step 0 completed')

%% Step 1: Load and visualise the four 2D images
%the path of the 2Ddata folder in your computer
str_datafolder = '../data/2Ddata/'; %default value

%Visualise all four datasets
contents_datafolder = dir(str_datafolder);
num_datasets = length(contents_datafolder)-2; 

figure
 for k= 1:num_datasets
    %read image
    im = imread([str_datafolder,contents_datafolder(k+2).name]); 
    %create subtitle for image
    subtitle = [num2str(k),'. ',contents_datafolder(k+2).name(1:end-4)];
    ind = find(subtitle=='_');
    subtitle(ind) = ' ';
    %plot image
    subplot(2,2,k), imagesc(im), axis image, colormap gray, title(subtitle)
 end
 
h = msgbox('Inspect the quality of the four different scans');
waitfor(h)

disp('Step 1 completed')

%% Step 2: Select data-set and region of interest (RoI)
close all
%USER INPUT: select dataset
x = inputdlg('Choose a data-set (1, 2, 3 or 4?): ','User input',[1,20],{num2str(4)});

%read the 2D image chosen by the user 
dataset = str2double(x{1}); 
im = imread([str_datafolder,contents_datafolder(dataset+2).name]);

%print the name of the selected data-set
disp(['Selected data-set: ', contents_datafolder(dataset+2).name(1:end-4)]);

%USER INPUT: select RoI
h = msgbox('Crop a region of interest containing 100-1000 fibres.'); 
waitfor(h)
im_crop = imcrop(im);

%visualise the selected RoI
if isempty(im_crop)
   h = msgbox('There was an error in the cropping. Run the section again.'); 
   waitfor(h)
else
    figure, imagesc(im_crop), axis image, colormap gray, 
    title('Region of Interest for training the dictionary')
end

disp('Step 2 completed')

%% Step 3: Set default parameters and the patch size at the scale of the fibres
close all
%USER INPUT: Parameters that give the scale of the fibres in pixels
prompt_px = ['Pixel size [', char(181),'m]: '];
prompt_diam = ['Diameter size [', char(181),'m]: '];
x = str2double(inputdlg({prompt_px, prompt_diam},'User input',[1,30],{num2str(0.96),num2str(7)}));

%compute patch size as the closest odd integer to the fibre diameter
%measured in pixels diam/pixel_size
patch_size = calcu_psize(x(2)/x(1));

%check that the patch size is in the optimal range to ensure precision and
%speed
if ((x(2)/x(1)) < 7)
    factor = 9/(x(2)/x(1));
    patch_size = calcu_psize(factor*x(2)/x(1));
    msg = ['We will upscale your images with a factor ',...
        num2str(factor,'%0.2f'),' to obtain better precision. The patch size is now ',...
        num2str(patch_size),'.'];
    h = msgbox(msg);
    waitfor(h)
elseif ((x(2)/x(1)) >= 17)
     factor = 11/(x(2)/x(1));
    patch_size = calcu_psize(factor*x(2)/x(1));
    msg = ['We will downscale your images with a factor ',...
        num2str(factor),' to obtain reduce the computational time. The patch size is now ',...
        num2str(patch_size),'.'];
    h = msgbox(msg);
    waitfor(h)
else
    factor = 1;
end

%set patch size
dictopt.patch_size = patch_size; 
%set the default ditionary parameters
dictopt.method = 'euclidean';
dictopt.branching_factor = 3; %it should at least be 3
dictopt.number_layers = 5; %at least 5. The higher the more accurate, but also slower and more computationally heavy
num_dict_elts = calc_eltsdict(dictopt.branching_factor,dictopt.number_layers);
dictopt.number_training_patches = 5000; %at least an order of magnitude more than the number of dictionary elements
dictopt.normalization = false; %set to true if the global intensity of the slices varies along the depth of the volume

disp('Step 3 completed')

%% Step 4: Run InSegt
%display the important dictionary parameters
fprintf('\nImportant Dictionary Parameters:\n');
disp(['patch size: ', num2str(dictopt.patch_size)]);
disp(['number dictionary elements: ', num2str(calc_eltsdict(dictopt.branching_factor,dictopt.number_layers))]);

%resize cropped region of interest with factor
im_train = imresize(im_crop, factor);
%open the GUI with 2 labels for pixel annotation (fibre centre region or not)
image_texture_gui(im_train,dictopt,2)

disp('Step 4 completed')

%% Step 5: Obtain the centre coordinates from the hard segmentation
close all
%USER INPUT: label used for the centre regions
label = 2; %1 (blue) or 2 (pink)

%calculate the centroids of the connected centre regions
c = regionprops(gui_S==label,'centroid');
centrePoints = cat(1,c.Centroid);

%show fibre centre coordinates over slice
figure(1), subplot(1,2,1), imagesc(im_crop), axis image, colormap gray, 
hold on, h = imagesc(ind2rgb(imresize(gui_S==label,1/factor), cool(2))); set(h, 'AlphaData',0.5);

figure(1), subplot(1,2,2), imagesc(im_crop), axis image, colormap gray
hold on, plot(centrePoints(:,1)*1/factor,centrePoints(:,2)*1/factor,'r*');

disp('Step 5 completed')

%% Step 6: Save the dictionary to process other regions of the scan
close all
%train the dictionary with the small RoI
dictionary = build_dictionary(im_train,dictopt); %create the dictionary of intensities
image_texture_gui(im_train,dictionary,2) %learn the dictionary of probabilities by annotating in the GUI 
dictionary = update_dictionary(dictionary,gui_dictprob); %update dictionary to include the learnt probalilities

%USER INPUT: select a RoI to process with the learnt dictionary
h = msgbox('Crop a region of interest, it can be as big as you like.'); 
waitfor(h)
im_p = imcrop(im);
if isempty(im_crop)
   h = msgbox('There was an error in the cropping. Run the section again from the cropping.'); 
   waitfor(h)
else
    figure, imagesc(im_p), axis image, colormap gray, 
    title('Region of Interest for finding fibre centres')
end

%apply the dictionary to process the larger RoI
im_process = imresize(im_p,factor);
[S,allP] = process_image(im_process,dictionary);
    
%calculate the centroids of the connected centre regions
label = 2; %1 (blue) or 2 (pink)
c = regionprops(S==label,'centroid');
centrePoints = cat(1,c.Centroid);

%Centre regions and coordinates over the image to evaluate the accuracy of the segmentation
figure, imagesc(im_process), axis image, colormap gray, 
hold on, h = imagesc(ind2rgb(S, cool(2))); set(h, 'AlphaData',0.5);
hold on, plot(centrePoints(:,1),centrePoints(:,2),'y.');
title('Segmented centre regions and fibre detections over the processed image');
    
if factor ~=1
    figure, imagesc(im_p), axis image, colormap gray, 
    hold on, plot(centrePoints(:,1)*1/factor,centrePoints(:,2)*1/factor,'y.');
    title('Centre detections over the original resolution')
end
    
disp('Step 6 completed')

%% Step 7: Tune the threshold applied to the probability map
close all
%visualise the propabilistic segmentation of fibre centre regions
figure(1), imagesc(allP(:,:,2)), axis image off, colormap gray, title('Centre class probability map'), colorbar

y = 'yes';
while strcmp(y,'yes') 
    %USER INPUT: threshold value for the probability map 
    x = inputdlg('Choose threshold: ','User input',[1,20],{num2str(0.5)});

    %Centre regions over the image to evaluate the accuracy of the segmentation
    figure, imagesc(im_process), axis image, colormap gray, 
    hold on, h = imagesc(ind2rgb(allP(:,:,2)>str2double(x{1}), cool(2))); set(h, 'AlphaData',0.5);
    
    %USER INPUT: do you want to try another threshold?? 
    y = questdlg('Do you want to try another thresold?: ','User input','yes','no','yes');
end

disp('Step 7 completed')

%% Step 8: Save your manual input to add additional markings later on
close all
%annotate a little in the GUI and save your manual input
h = msgbox(['Annotate a little in the GUI and press [S] before closing the GUI (then close by pressing export [E]). Choose to save',...
    ' your annotations in the folder "InsegtFibre_workshopDTU/data/saved_data/trainingData".']); 
waitfor(h)
image_texture_gui(im_train,dictopt,2)  
%save the training region, over which you placed the markings
save('../data/saved_data/trainingData/training_image.mat','im_train');

%Call the GUI with a saved labelling
load('../data/saved_data/trainingData/training_image.mat');
[filename,pathname] = uigetfile('*_labels_indexed.png', 'Choose label image', '../data/saved_data/trainingData/');  
labelling = imread([pathname,filename]); %read labelling
image_texture_gui(im_train,dictopt,2,labelling) %call the GUI with the saved labelling

disp('Step 8 completed')
