% InSegtFibre_orientations
% Written by Monica Jane Emerson, May 2019.
% MIT license

%% Prepare workspace
close all
clear all
addpath('./scripts')
addpath('./functions')

%% Read tracks, z-values (analysed slices) and voxel size 
[filename_tracks,pathname_tracks] = uigetfile('*.mat','Choose tracks','../data/saved_data/fibreTracks');
fibres = getfield(load([pathname_tracks,filename_tracks]),'fibres');

depth = input_depthRoI; %get z coordinates (start, end and step)
z_axis = [str2double(depth{1}):str2double(depth{3}):str2double(depth{2})];

vx_size = str2double(inputdlg(['What is the pixel size of your scan [', char(181),'m]: '],'Pixel/voxel size',[1,45],{num2str(1.1)}));

%% Display tracks
h1 = figure(1);
for l= 1:size(fibres.x,1)
    figure(1), hold on, 
    plot3(fibres.x(l,:)*vx_size,fibres.y(l,:)*vx_size,z_axis*vx_size,'k','LineWidth',1), 
end
figure(1), axis tight, axis equal, view(2), view(3) 
xlabel('x axis [\mum]'),ylabel('y axis [\mum]'), zlabel('z axis [\mum]')

%% Display locations of missed detections
missed_det = isnan(fibres.x');
figure(2), imagesc(missed_det), axis image, axis xy
title('Missed fibre centres')
ylabel('z axis [slices]'),xlabel('fibre ID'),

%% Select fibres for the orientation plot
%Indices of fibres that miss start and/or end detection
%(will be excluded from the orientation plot)
exc_fibres = find((missed_det(1,:)|missed_det(end,:))==1);

%Indices of the fibre kept for the orientation plot
keep_fibres = 1:size(fibres.x,1);
keep_fibres(exc_fibres) = []; 

display(['The orientation can been computed for ',num2str(length(keep_fibres)/size(fibres.x,1)*100,'%.2f') ,'% of the tracked fibres']);

%% Compute orientation (inclination and it's direction, azimuth)
%displacement between end and start points
disp_x = fibres.x(keep_fibres,end)-fibres.x(keep_fibres,1);
disp_y = fibres.y(keep_fibres,end)-fibres.y(keep_fibres,1);
disp_r = sqrt(disp_x.^2 + disp_y.^2);

%inclination and azimuth angles
inclination = atan2(disp_r,(size(fibres.x,2)-1)*ones(length(keep_fibres),1));
azimuth = atan2(disp_y,disp_x);

display('The orientations (inclination and azimuth angles) have been computed');

%% Histograms of orientation angles
close all
%Inclination
max_inc = str2double(inputdlg('What is the maximum misalignment angle you expect [0, 90]?','Max. Inclination',[1,35],{num2str(15)}));
bins_inc = [0:max_inc];
int_inc = [0:0.1:max_inc];

h_inc = hist(inclination*180/pi,bins_inc)/length(inclination);
h_inc_int = interp1(bins_inc,h_inc,int_inc);
h1 = figure(1),hold on, plot(int_inc,smooth(h_inc_int))
ylim([0,max(smooth(h_inc_int))]), xlim([0,max_inc]),h1.Children.XTick = linspace(0,max_inc,10);
xlabel('Inclination \phi [\circ] of the fibres'), ylabel('Relative frequency')
title('Amount of fibre misalignment with respect to the scanning z-axis'),

%Azimuth
max_az = 360;
bins_az = [-max_az/2:24:max_az/2];
int_az = [-max_az/2:2.4:max_az/2];

h_az = hist(azimuth*180/pi,bins_az)/length(azimuth);
h_az_int = interp1(bins_az,h_az,int_az);
h2 = figure(2),hold on, plot(int_az,smooth(h_az_int))
xlim([-max_az/2,max_az/2]), h2.Children.XTick = [-180,-90,0,90,180];
ylim([0,max(smooth(h_az_int))])
xlabel('Azimuth \theta [\circ] of the fibres'), ylabel('Relative frequency')
title('Direction of misalignment with respect to scanning x-axis'),

%% Colour-code fibre tracks according to orientation angles
%Azimuth (direction of misalingment)
cmap_az = circshift(colormap(hsv),32);
h3 = figure(3);
for l= 1:length(keep_fibres)
    colouring = cmap_az(round(azimuth(l)/(2*pi)*63+1+63/2),:);
    figure(3), hold on, plot3(fibres.x(keep_fibres(l),:)*vx_size,fibres.y(keep_fibres(l),:)*vx_size,z_axis*vx_size,'LineWidth',1,'Color',colouring), 
end
figure(3), axis tight, axis equal, view(2), 
xlabel('x axis [\mum]'),ylabel('y axis [\mum]'), zlabel('z axis [\mum]')
colormap(cmap_az), colorbar, caxis([-180,180]), h3.Children(1).Ticks = -180:30:180;
for k = 1:length(h3.Children(1).Ticks)
    h3.Children(1).TickLabels(k) = {[h3.Children(1).TickLabels{k},'']};
end

%Inclination (amount of misalignment)
cmap_inc = linspace(0,1,32)'*[1,1,1];
sat_angle = max_inc/3;
sat = sat_angle*pi/180;
h4 = figure(4);
for l= 1:length(keep_fibres)
    if inclination(l) < sat
            colouring = [1,1,1]*inclination(l)/sat;
        else
            colouring = [1,1,1];
        end
    figure(4), hold on, plot3(fibres.x(keep_fibres(l),:)*vx_size,fibres.y(keep_fibres(l),:)*vx_size,z_axis*vx_size,'LineWidth',1,'Color',colouring), 
end
figure(4), axis tight, axis equal, view(2), 
xlabel('x axis [\mum]'),ylabel('y axis [\mum]'), zlabel('z axis [\mum]')
colormap(cmap_inc), 
colorbar, caxis([0,sat_angle]), h.Children(1).Ticks = linspace(0,sat_angle,5); 
h4.Children(1).TickLabels(end) = {['>',num2str(sat_angle)]};
for k = 1:length(h.Children(1).Ticks)
    h4.Children(1).TickLabels(k) = {[h4.Children(1).TickLabels{k},'']};
end

%Azimuth weighed by inclination (amount of misalignment)
h5 = figure(5);
for l= 1:length(keep_fibres)
    if inclination(l) < sat
            colouring = cmap_az(round(azimuth(l)/(2*pi)*63+1+63/2),:)*inclination(l)/sat;
        else
            colouring = cmap_az(round(azimuth(l)/(2*pi)*63+1+63/2),:);
        end
    figure(5), hold on, plot3(fibres.x(keep_fibres(l),:)*vx_size,fibres.y(keep_fibres(l),:)*vx_size,z_axis*vx_size,'LineWidth',1,'Color',colouring), 
end
figure(5), axis tight, axis equal, view(2), 
xlabel('x axis [\mum]'),ylabel('y axis [\mum]'), zlabel('z axis [\mum]')
colormap(cmap_az), colorbar, caxis([-180,180]), h5.Children(1).Ticks = -180:30:180;
for k = 1:length(h.Children(1).Ticks)
    h5.Children(1).TickLabels(k) = {[h5.Children(1).TickLabels{k},'']};
end

